@@ -36,8 +36,8 @@ type argContainer struct {
 	memprofile, ko, passfile, ctlsock, fsname, force_owner, trace string
 	// -extpass can be passed multiple times
 	extpass multipleStrings
-	// For reverse mode, -exclude is available. It can be specified multiple times.
-	exclude multipleStrings
+	// For reverse mode, several ways to specify exclusions. All can be specified multiple times.
+	exclude, excludeWildcard, excludeFrom multipleStrings
 	// Configuration file name override
 	config             string
 	notifypid, scryptn int
@@ -50,6 +50,8 @@ type argContainer struct {
 	_ctlsockFd net.Listener
 	// _forceOwner is, if non-nil, a parsed, validated Owner (as opposed to the string above)
 	_forceOwner *fuse.Owner
+	// _explicitScryptn is true then the user passed "-scryptn=xyz"
+	_explicitScryptn bool
 }
 
 type multipleStrings []string
@@ -193,21 +195,29 @@ func parseCliOpts() (args argContainer) {
 	flagSet.StringVar(&args.force_owner, "force_owner", "", "uid:gid pair to coerce ownership")
 	flagSet.StringVar(&args.trace, "trace", "", "Write execution trace to file")
 
-	// -e, --exclude
+	// Exclusion options
 	flagSet.Var(&args.exclude, "e", "Alias for -exclude")
 	flagSet.Var(&args.exclude, "exclude", "Exclude relative path from reverse view")
+	flagSet.Var(&args.excludeWildcard, "ew", "Alias for -exclude-wildcard")
+	flagSet.Var(&args.excludeWildcard, "exclude-wildcard", "Exclude path from reverse view, supporting wildcards")
+	flagSet.Var(&args.excludeFrom, "exclude-from", "File from which to read exclusion patterns (with -exclude-wildcard syntax)")
+
 	// -extpass
 	flagSet.Var(&args.extpass, "extpass", "Use external program for the password prompt")
 
 	flagSet.IntVar(&args.notifypid, "notifypid", 0, "Send USR1 to the specified process after "+
 		"successful mount - used internally for daemonization")
-	flagSet.IntVar(&args.scryptn, "scryptn", configfile.ScryptDefaultLogN, "scrypt cost parameter logN. Possible values: 10-28. "+
+	const scryptn = "scryptn"
+	flagSet.IntVar(&args.scryptn, scryptn, configfile.ScryptDefaultLogN, "scrypt cost parameter logN. Possible values: 10-28. "+
 		"A lower value speeds up mounting and reduces its memory needs, but makes the password susceptible to brute-force attacks")
 
 	flagSet.DurationVar(&args.idle, "i", 0, "Alias for -idle")
 	flagSet.DurationVar(&args.idle, "idle", 0, "Auto-unmount after specified idle duration (ignored in reverse mode). "+
 		"Durations are specified like \"500s\" or \"2h45m\". 0 means stay mounted indefinitely.")
 
+	var nofail bool
+	flagSet.BoolVar(&nofail, "nofail", false, "Ignored for /etc/fstab compatibility")
+
 	var dummyString string
 	flagSet.StringVar(&dummyString, "o", "", "For compatibility with mount(1), options can be also passed as a comma-separated list to -o on the end.")
 	// Actual parsing
@@ -220,6 +230,10 @@ func parseCliOpts() (args argContainer) {
 		tlog.Fatal.Printf("Invalid command line: %s. Try '%s -help'.", prettyArgs(), tlog.ProgramName)
 		os.Exit(exitcodes.Usage)
 	}
+	// We want to know if -scryptn was passed explicitely
+	if isFlagPassed(flagSet, scryptn) {
+		args._explicitScryptn = true
+	}
 	// "-openssl" needs some post-processing
 	if opensslAuto == "auto" {
 		args.openssl = prefer_openssl.PreferOpenSSL()
@@ -304,3 +318,15 @@ func countOpFlags(args *argContainer) int {
 	}
 	return count
 }
+
+// isFlagPassed finds out if the flag was explictely passed on the command line.
+// https://stackoverflow.com/a/54747682/1380267
+func isFlagPassed(flagSet *flag.FlagSet, name string) bool {
+	found := false
+	flagSet.Visit(func(f *flag.Flag) {
+		if f.Name == name {
+			found = true
+		}
+	})
+	return found
+}
