@@ -151,7 +151,7 @@ func concatAD(blockNo uint64, fileID []byte) (aData []byte) {
 // DecryptBlock - Verify and decrypt GCM block
 //
 // Corner case: A full-sized block of all-zero ciphertext bytes is translated
-// to an all-zero plaintext block, i.e. file hole passtrough.
+// to an all-zero plaintext block, i.e. file hole passthrough.
 func (be *ContentEnc) DecryptBlock(ciphertext []byte, blockNo uint64, fileID []byte) ([]byte, error) {
 
 	// Empty block?
@@ -324,3 +324,10 @@ func (be *ContentEnc) MergeBlocks(oldData []byte, newData []byte, offset int) []
 	}
 	return out[0:outLen]
 }
+
+// Wipe tries to wipe secret keys from memory by overwriting them with zeros
+// and/or setting references to nil.
+func (be *ContentEnc) Wipe() {
+	be.cryptoCore.Wipe()
+	be.cryptoCore = nil
+}
