@@ -62,7 +62,7 @@ func (f *File) Allocate(off uint64, sz uint64, mode uint32) fuse.Status {
 	// the file.
 	cipherOff := firstBlock.BlockCipherOff()
 	cipherSz := lastBlock.BlockCipherOff() - cipherOff +
-		f.contentEnc.PlainSizeToCipherSize(lastBlock.Skip+lastBlock.Length)
+		f.contentEnc.BlockOverhead() + lastBlock.Skip + lastBlock.Length
 	err := syscallcompat.Fallocate(f.intFd(), FALLOC_FL_KEEP_SIZE, int64(cipherOff), int64(cipherSz))
 	tlog.Debug.Printf("Allocate off=%d sz=%d mode=%x cipherOff=%d cipherSz=%d\n",
 		off, sz, mode, cipherOff, cipherSz)
@@ -181,18 +181,17 @@ func (f *File) truncateGrowFile(oldPlainSz uint64, newPlainSz uint64) fuse.Statu
 	if newPlainSz <= oldPlainSz {
 		log.Panicf("BUG: newSize=%d <= oldSize=%d", newPlainSz, oldPlainSz)
 	}
-	var n1 uint64
-	if oldPlainSz > 0 {
-		n1 = f.contentEnc.PlainOffToBlockNo(oldPlainSz - 1)
-	}
 	newEOFOffset := newPlainSz - 1
-	n2 := f.contentEnc.PlainOffToBlockNo(newEOFOffset)
-	// The file is grown within one block, no need to pad anything.
-	// Write a single zero to the last byte and let doWrite figure out the RMW.
-	if n1 == n2 {
-		buf := make([]byte, 1)
-		_, status := f.doWrite(buf, int64(newEOFOffset))
-		return status
+	if oldPlainSz > 0 {
+		n1 := f.contentEnc.PlainOffToBlockNo(oldPlainSz - 1)
+		n2 := f.contentEnc.PlainOffToBlockNo(newEOFOffset)
+		// The file is grown within one block, no need to pad anything.
+		// Write a single zero to the last byte and let doWrite figure out the RMW.
+		if n1 == n2 {
+			buf := make([]byte, 1)
+			_, status := f.doWrite(buf, int64(newEOFOffset))
+			return status
+		}
 	}
 	// The truncate creates at least one new block.
 	//
