@@ -6,6 +6,7 @@ package fusefrontend
 import (
 	"os"
 	"sync"
+	"sync/atomic"
 	"syscall"
 	"time"
 
@@ -35,7 +36,7 @@ type FS struct {
 	// states
 	dirIVLock sync.RWMutex
 	// Filename encryption helper
-	nameTransform *nametransform.NameTransform
+	nameTransform nametransform.NameTransformer
 	// Content encryption helper
 	contentEnc *contentenc.ContentEnc
 	// This lock is used by openWriteOnlyFile() to block concurrent opens while
@@ -50,11 +51,11 @@ type FS struct {
 	// "gocryptfs -fsck" reads from the channel to also catch these transparently-
 	// mitigated corruptions.
 	MitigatedCorruptions chan string
-	// Track accesses to the filesystem so that we can know when to autounmount.
-	// An access is considered to have happened on every call to encryptPath,
-	// which is called as part of every filesystem operation.
-	// (This flag uses a uint32 so that it can be reset with CompareAndSwapUint32.)
-	AccessedSinceLastCheck uint32
+	// This flag is set to zero each time fs.isFiltered() is called
+	// (uint32 so that it can be reset with CompareAndSwapUint32).
+	// When -idle was used when mounting, idleMonitor() sets it to 1
+	// periodically.
+	IsIdle uint32
 
 	dirCache dirCacheStruct
 }
@@ -62,7 +63,7 @@ type FS struct {
 //var _ pathfs.FileSystem = &FS{} // Verify that interface is implemented.
 
 // NewFS returns a new encrypted FUSE overlay filesystem.
-func NewFS(args Args, c *contentenc.ContentEnc, n *nametransform.NameTransform) *FS {
+func NewFS(args Args, c *contentenc.ContentEnc, n nametransform.NameTransformer) *FS {
 	if args.SerializeReads {
 		serialize_reads.InitSerializer()
 	}
@@ -399,7 +400,7 @@ func (fs *FS) decryptSymlinkTarget(cData64 string) (string, error) {
 	if cData64 == "" {
 		return "", nil
 	}
-	cData, err := fs.nameTransform.B64.DecodeString(cData64)
+	cData, err := fs.nameTransform.B64DecodeString(cData64)
 	if err != nil {
 		return "", err
 	}
@@ -472,7 +473,7 @@ func (fs *FS) encryptSymlinkTarget(data string) (cData64 string) {
 		return ""
 	}
 	cData := fs.contentEnc.EncryptBlock([]byte(data), 0, nil)
-	cData64 = fs.nameTransform.B64.EncodeToString(cData)
+	cData64 = fs.nameTransform.B64EncodeToString(cData)
 	return cData64
 }
 
@@ -662,6 +663,8 @@ func (fs *FS) reportMitigatedCorruption(item string) {
 //
 // Prevents name clashes with internal files when file names are not encrypted
 func (fs *FS) isFiltered(path string) bool {
+	atomic.StoreUint32(&fs.IsIdle, 0)
+
 	if !fs.args.PlaintextNames {
 		return false
 	}
