@@ -19,6 +19,8 @@ import (
 	"syscall"
 	"time"
 
+	"golang.org/x/sys/unix"
+
 	"github.com/hanwen/go-fuse/fuse"
 	"github.com/hanwen/go-fuse/fuse/nodefs"
 	"github.com/hanwen/go-fuse/fuse/pathfs"
@@ -62,7 +64,7 @@ func doMount(args *argContainer) {
 	if args.nonempty {
 		err = isDir(args.mountpoint)
 	} else {
-		err = isDirEmpty(args.mountpoint)
+		err = isEmptyDir(args.mountpoint)
 		// OSXFuse will create the mountpoint for us ( https://github.com/rfjakob/gocryptfs/issues/194 )
 		if runtime.GOOS == "darwin" && os.IsNotExist(err) {
 			tlog.Info.Printf("Mountpoint %q does not exist, but should be created by OSXFuse",
@@ -95,6 +97,21 @@ func doMount(args *argContainer) {
 			}
 		}()
 	}
+	// Preallocation on Btrfs is broken ( https://github.com/rfjakob/gocryptfs/issues/395 )
+	// and slow ( https://github.com/rfjakob/gocryptfs/issues/63 ).
+	if !args.noprealloc {
+		// darwin does not have unix.BTRFS_SUPER_MAGIC, so we define it here
+		const BTRFS_SUPER_MAGIC = 0x9123683e
+		var st unix.Statfs_t
+		err = unix.Statfs(args.cipherdir, &st)
+		// Cast to uint32 avoids compile error on arm: "constant 2435016766 overflows int32"
+		if err == nil && uint32(st.Type) == BTRFS_SUPER_MAGIC {
+			tlog.Info.Printf(tlog.ColorYellow +
+				"Btrfs detected, forcing -noprealloc. See https://github.com/rfjakob/gocryptfs/issues/395 for why." +
+				tlog.ColorReset)
+			args.noprealloc = true
+		}
+	}
 	// We cannot use JSON for pretty-printing as the fields are unexported
 	tlog.Debug.Printf("cli args: %#v", args)
 	// Initialize gocryptfs (read config file, ask for password, ...)
@@ -164,18 +181,18 @@ func idleMonitor(idleTimeout time.Duration, fs *fusefrontend.FS, srv *fuse.Serve
 	timeoutCycles := int(math.Ceil(float64(idleTimeout) / float64(sleepTimeBetweenChecks)))
 	idleCount := 0
 	for {
-		// Atomically check whether the access flag is set and reset it to 0 if so.
-		recentAccess := atomic.CompareAndSwapUint32(&fs.AccessedSinceLastCheck, 1, 0)
+		// Atomically check whether the flag is 0 and reset it to 1 if so.
+		isIdle := !atomic.CompareAndSwapUint32(&fs.IsIdle, 0, 1)
 		// Any form of current or recent access resets the idle counter.
 		openFileCount := openfiletable.CountOpenFiles()
-		if recentAccess || openFileCount > 0 {
+		if !isIdle || openFileCount > 0 {
 			idleCount = 0
 		} else {
 			idleCount++
 		}
 		tlog.Debug.Printf(
-			"Checking for idle (recentAccess = %t, open = %d): %s",
-			recentAccess, openFileCount, time.Now().String())
+			"Checking for idle (isIdle = %t, open = %d): %s",
+			isIdle, openFileCount, time.Now().String())
 		if idleCount > 0 && idleCount%timeoutCycles == 0 {
 			tlog.Info.Printf("Filesystem idle; unmounting: %s", mountpoint)
 			unmount(srv, mountpoint)
@@ -231,15 +248,17 @@ func initFuseFrontend(args *argContainer) (pfs pathfs.FileSystem, wipeKeys func(
 		args.allow_other = true
 	}
 	frontendArgs := fusefrontend.Args{
-		Cipherdir:      args.cipherdir,
-		PlaintextNames: args.plaintextnames,
-		LongNames:      args.longnames,
-		ConfigCustom:   args._configCustom,
-		NoPrealloc:     args.noprealloc,
-		SerializeReads: args.serialize_reads,
-		ForceDecode:    args.forcedecode,
-		ForceOwner:     args._forceOwner,
-		Exclude:        args.exclude,
+		Cipherdir:       args.cipherdir,
+		PlaintextNames:  args.plaintextnames,
+		LongNames:       args.longnames,
+		ConfigCustom:    args._configCustom,
+		NoPrealloc:      args.noprealloc,
+		SerializeReads:  args.serialize_reads,
+		ForceDecode:     args.forcedecode,
+		ForceOwner:      args._forceOwner,
+		Exclude:         args.exclude,
+		ExcludeWildcard: args.excludeWildcard,
+		ExcludeFrom:     args.excludeFrom,
 	}
 	// confFile is nil when "-zerokey" or "-masterkey" was used
 	if confFile != nil {
