@@ -7,6 +7,7 @@ import (
 	"io/ioutil"
 	"os"
 	"os/exec"
+	"strconv"
 	"syscall"
 	"testing"
 	"time"
@@ -21,11 +22,11 @@ var testPw = []byte("test")
 
 func TestMain(m *testing.M) {
 	test_helpers.ResetTmpDir(false)
-	before := test_helpers.ListFds(0)
+	before := test_helpers.ListFds(0, "")
 	r := m.Run()
-	after := test_helpers.ListFds(0)
+	after := test_helpers.ListFds(0, "")
 	if len(before) != len(after) {
-		fmt.Printf("fd leak? before, after:\n%v\n%v\n", before, after)
+		fmt.Printf("fd leak in test process? before, after:\n%v\n%v\n", before, after)
 		os.Exit(1)
 	}
 	os.Exit(r)
@@ -43,6 +44,26 @@ func TestInit(t *testing.T) {
 	}
 }
 
+// Test that gocryptfs.conf and gocryptfs.diriv are there with the expected
+// permissions after -init
+func TestInitFilePerms(t *testing.T) {
+	dir := test_helpers.InitFS(t)
+	var st syscall.Stat_t
+	syscall.Stat(dir+"/gocryptfs.conf", &st)
+	perms := st.Mode & 0777
+	if perms != 0400 {
+		t.Errorf("Wrong permissions for gocryptfs.conf: %#o", perms)
+	}
+	st = syscall.Stat_t{}
+	syscall.Stat(dir+"/gocryptfs.diriv", &st)
+	perms = st.Mode & 0777
+	// From v1.7.1, these are created with 0440 permissions, see
+	// https://github.com/rfjakob/gocryptfs/issues/387
+	if perms != 0440 {
+		t.Errorf("Wrong permissions for gocryptfs.diriv: %#o", perms)
+	}
+}
+
 // Test -init with -devrandom flag
 func TestInitDevRandom(t *testing.T) {
 	test_helpers.InitFS(t, "-devrandom")
@@ -72,6 +93,9 @@ func TestInitReverse(t *testing.T) {
 	}
 }
 
+// testPasswd changes the password from "test" to "test" using
+// the -extpass method, then from "test" to "newpasswd" using the
+// stdin method.
 func testPasswd(t *testing.T, dir string, extraArgs ...string) {
 	// Change password using "-extpass"
 	args := []string{"-q", "-passwd", "-extpass", "echo test"}
@@ -205,6 +229,23 @@ func TestPasswdReverse(t *testing.T) {
 	testPasswd(t, dir, "-reverse")
 }
 
+// Test -passwd with -scryptn
+func TestPasswdScryptn(t *testing.T) {
+	dir := test_helpers.InitFS(t)
+	cf, err := configfile.Load(dir + "/gocryptfs.conf")
+	if err != nil {
+		t.Fatal(err)
+	}
+	testPasswd(t, dir, "-scryptn", strconv.Itoa(cf.ScryptObject.LogN()+1))
+	cf2, err := configfile.Load(dir + "/gocryptfs.conf")
+	if err != nil {
+		t.Fatal(err)
+	}
+	if cf2.ScryptObject.LogN() != cf.ScryptObject.LogN()+1 {
+		t.Errorf("wrong logN value %d", cf2.ScryptObject.LogN())
+	}
+}
+
 // Test -init & -config flag
 func TestInitConfig(t *testing.T) {
 	config := test_helpers.TmpDir + "/TestInitConfig.conf"
@@ -265,6 +306,14 @@ func TestNonempty(t *testing.T) {
 	test_helpers.UnmountPanic(mnt)
 }
 
+// -nofail should be ignored and the mount should succeed
+func TestNofail(t *testing.T) {
+	dir := test_helpers.InitFS(t)
+	mnt := dir + ".mnt"
+	test_helpers.MountOrFatal(t, dir, mnt, "-nofail", "-extpass=echo test")
+	defer test_helpers.UnmountPanic(mnt)
+}
+
 // Test "mountpoint shadows cipherdir" handling
 func TestShadows(t *testing.T) {
 	mnt := test_helpers.InitFS(t)
@@ -289,50 +338,6 @@ func TestShadows(t *testing.T) {
 	}
 }
 
-// TestInitTrailingGarbage verfies that gocryptfs exits with an error if we
-// pass additional data after the password.
-func TestInitTrailingGarbage(t *testing.T) {
-	table := []struct {
-		pw            string
-		closeStdin    bool
-		expectSuccess bool
-	}{
-		{"foo\n", false, true},
-		{"foo", true, true},
-		{"foo\n", true, true},
-		{"foo\n\n", false, false},
-		{"foo\nbar", false, false},
-		{"foo\n\n", true, false},
-		{"foo\nbar", true, false},
-	}
-	for _, row := range table {
-		dir, err := ioutil.TempDir(test_helpers.TmpDir, "")
-		if err != nil {
-			t.Fatal(err)
-		}
-		cmd := exec.Command(test_helpers.GocryptfsBinary, "-q", "-init", "-scryptn=10", dir)
-		childStdin, err := cmd.StdinPipe()
-		if err != nil {
-			t.Fatal(err)
-		}
-		err = cmd.Start()
-		if err != nil {
-			t.Fatal(err)
-		}
-		childStdin.Write([]byte(row.pw))
-		if row.closeStdin {
-			childStdin.Close()
-		}
-		err = cmd.Wait()
-		success := (err == nil)
-		if success == true && row.expectSuccess == false {
-			t.Errorf("pw=%q should have failed, but succeeded", row.pw)
-		} else if success == false && row.expectSuccess == true {
-			t.Errorf("pw=%q should have succeeded, but failed", row.pw)
-		}
-	}
-}
-
 // TestMountPasswordIncorrect makes sure the correct exit code is used when the password
 // was incorrect while mounting
 func TestMountPasswordIncorrect(t *testing.T) {
@@ -563,3 +568,44 @@ func TestIdle(t *testing.T) {
 		t.Error(err)
 	}
 }
+
+// Mount with idle timeout of 100ms read something every 10ms. The fs should
+// NOT get unmounted. Regression test for https://github.com/rfjakob/gocryptfs/issues/421
+func TestNotIdle(t *testing.T) {
+	dir := test_helpers.InitFS(t)
+	mnt := dir + ".mnt"
+	err := os.Mkdir(mnt, 0700)
+	if err != nil {
+		t.Fatal(err)
+	}
+	err = test_helpers.Mount(dir, mnt, false, "-extpass", "echo test", "-i=100ms")
+	if err != nil {
+		t.Fatal(err)
+	}
+	err = ioutil.WriteFile(mnt+"/foo", []byte("foo"), 0600)
+	if err != nil {
+		t.Fatal(err)
+	}
+	// Read every 10 milliseconds for a total of 1 second
+	for i := 1; i < 100; i++ {
+		_, err = ioutil.ReadFile(mnt + "/foo")
+		if err != nil {
+			t.Fatalf("iteration %d failed: %v", i, err)
+		}
+		time.Sleep(10 * time.Millisecond)
+	}
+	// Keep a file handle open for 1 second
+	fd, err := os.Open(mnt + "/foo")
+	if err != nil {
+		t.Fatal(err)
+	}
+	time.Sleep(1 * time.Second)
+	buf := make([]byte, 100)
+	_, err = fd.Read(buf)
+	if err != nil {
+		t.Fatal(err)
+	}
+	fd.Close()
+	// All good.
+	test_helpers.UnmountPanic(mnt)
+}
