@@ -38,21 +38,24 @@ type testcaseMatrix struct {
 	openssl        string
 	aessiv         bool
 	raw64          bool
+	extraArgs      []string
 }
 
 var matrix = []testcaseMatrix{
 	// Normal
-	{false, "auto", false, false},
-	{false, "true", false, false},
-	{false, "false", false, false},
+	{false, "auto", false, false, nil},
+	{false, "true", false, false, nil},
+	{false, "false", false, false, nil},
 	// Plaintextnames
-	{true, "true", false, false},
-	{true, "false", false, false},
+	{true, "true", false, false, nil},
+	{true, "false", false, false, nil},
 	// AES-SIV (does not use openssl, no need to test permutations)
-	{false, "auto", true, false},
-	{true, "auto", true, false},
+	{false, "auto", true, false, nil},
+	{true, "auto", true, false, nil},
 	// Raw64
-	{false, "auto", false, true},
+	{false, "auto", false, true, nil},
+	// -serialize_reads
+	{false, "auto", false, false, []string{"-serialize_reads"}},
 }
 
 // This is the entry point for the tests
@@ -72,14 +75,17 @@ func TestMain(m *testing.M) {
 		opts = append(opts, fmt.Sprintf("-plaintextnames=%v", testcase.plaintextnames))
 		opts = append(opts, fmt.Sprintf("-aessiv=%v", testcase.aessiv))
 		opts = append(opts, fmt.Sprintf("-raw64=%v", testcase.raw64))
+		opts = append(opts, testcase.extraArgs...)
 		test_helpers.MountOrExit(test_helpers.DefaultCipherDir, test_helpers.DefaultPlainDir, opts...)
-		before := test_helpers.ListFds(0)
+		before := test_helpers.ListFds(0, test_helpers.TmpDir)
 		r := m.Run()
 		// Catch fd leaks in the tests. NOTE: this does NOT catch leaks in
 		// the gocryptfs FUSE process, but only in the tests that access it!
-		after := test_helpers.ListFds(0)
+		// All fds that point outside TmpDir are not interesting (the Go test
+		// infrastucture creates temporary log files we don't care about).
+		after := test_helpers.ListFds(0, test_helpers.TmpDir)
 		if len(before) != len(after) {
-			fmt.Printf("fd leak? before, after:\n%v\n%v\n", before, after)
+			fmt.Printf("fd leak in test process? before, after:\n%v\n%v\n", before, after)
 			os.Exit(1)
 		}
 		test_helpers.UnmountPanic(test_helpers.DefaultPlainDir)
@@ -147,6 +153,45 @@ func TestWrite100x100(t *testing.T) {
 	}
 }
 
+func TestWrite10Tight(t *testing.T) {
+	path := test_helpers.DefaultPlainDir + "/TestWrite10Tight"
+	content := make([]byte, 10)
+	buf := make([]byte, 100)
+	for i := 0; i < 100; i++ {
+		file, err := os.Create(path)
+		if err != nil {
+			t.Fatal(err)
+		}
+		_, err = file.Write(content)
+		if err != nil {
+			t.Fatal(err)
+		}
+		err = file.Close()
+		if err != nil {
+			t.Fatal(err)
+		}
+		file, err = os.Open(path)
+		if err != nil {
+			t.Fatal(err)
+		}
+		n, err := file.Read(buf)
+		if err != nil {
+			t.Fatal(err)
+		}
+		if n != 10 {
+			t.Fatalf("want 10 bytes, got %d", n)
+		}
+		err = file.Close()
+		if err != nil {
+			t.Fatal(err)
+		}
+		err = os.Remove(path)
+		if err != nil {
+			t.Fatal()
+		}
+	}
+}
+
 // Hint for calculating reference md5sums:
 // dd if=/dev/zero count=1 bs=XYZ | md5sum
 func TestTruncate(t *testing.T) {
