@@ -44,14 +44,6 @@ var DefaultPlainDir string
 // DefaultCipherDir is TmpDir + "/default-cipher"
 var DefaultCipherDir string
 
-// SwitchTMPDIR changes TMPDIR and hence the directory the test are performed in.
-// This is used when you want to perform tests on a special filesystem. The
-// xattr tests cannot run on tmpfs and use /var/tmp instead of /tmp.
-func SwitchTMPDIR(newDir string) {
-	os.Setenv("TMPDIR", newDir)
-	doInit()
-}
-
 func init() {
 	doInit()
 }
@@ -59,14 +51,16 @@ func init() {
 func doInit() {
 	X255 = string(bytes.Repeat([]byte("X"), 255))
 	MountInfo = make(map[string]mountInfo)
-
-	testParentDir := os.TempDir() + "/gocryptfs-test-parent"
-	os.MkdirAll(testParentDir, 0700)
+	// Something like /tmp/gocryptfs-test-parent-1234
+	testParentDir := fmt.Sprintf("%s/gocryptfs-test-parent-%d", os.TempDir(), os.Getuid())
+	os.MkdirAll(testParentDir, 0755)
 	var err error
 	TmpDir, err = ioutil.TempDir(testParentDir, "")
 	if err != nil {
 		panic(err)
 	}
+	// Open permissions for the allow_other tests
+	os.Chmod(TmpDir, 0755)
 	DefaultPlainDir = TmpDir + "/default-plain"
 	DefaultCipherDir = TmpDir + "/default-cipher"
 }
@@ -104,11 +98,11 @@ func ResetTmpDir(createDirIV bool) {
 			}
 		}
 	}
-	err = os.Mkdir(DefaultPlainDir, 0700)
+	err = os.Mkdir(DefaultPlainDir, 0755)
 	if err != nil {
 		panic(err)
 	}
-	err = os.Mkdir(DefaultCipherDir, 0700)
+	err = os.Mkdir(DefaultCipherDir, 0755)
 	if err != nil {
 		panic(err)
 	}
@@ -125,10 +119,11 @@ func ResetTmpDir(createDirIV bool) {
 	}
 }
 
-// InitFS calls "gocryptfs -init" on a new directory in TmpDir, passing
-// "extraArgs" in addition to useful defaults.
+// InitFS creates a new empty cipherdir and calls
 //
-// The returned cipherdir has NO trailing slash.
+//     gocryptfs -q -init -extpass "echo test" -scryptn=10 $extraArgs $cipherdir
+//
+// It returns cipherdir without a trailing slash.
 func InitFS(t *testing.T, extraArgs ...string) string {
 	dir, err := ioutil.TempDir(TmpDir, "")
 	if err != nil {
@@ -149,7 +144,7 @@ func InitFS(t *testing.T, extraArgs ...string) string {
 	err = cmd.Run()
 	if err != nil {
 		if t != nil {
-			t.Fatalf("InitFS with args %v failed: %v", args, err)
+			t.Fatalf("InitFS with args %q failed: %v", args, err)
 		} else {
 			log.Panic(err)
 		}
