@@ -23,12 +23,6 @@ import (
 )
 
 func TestMain(m *testing.M) {
-	// On modern Linux distributions, /tmp may be on tmpfs,
-	// which does not support user xattrs. Try /var/tmp instead
-	if !xattrSupported(test_helpers.TmpDir) && os.TempDir() == "/tmp" {
-		fmt.Printf("Switching from /tmp to /var/tmp for xattr tests\n")
-		test_helpers.SwitchTMPDIR("/var/tmp")
-	}
 	if !xattrSupported(test_helpers.TmpDir) {
 		fmt.Printf("xattrs not supported on %q\n", test_helpers.TmpDir)
 		os.Exit(1)
